--*******************************************************************************************************************************
--
-- Description: This module controls which channel is active.
--					
-- Rev History: 08-22-2014:	Inital release.
--
--*******************************************************************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use IEEE.numeric_std.all;

use work.model_specific_package.all;
use work.transceiver_package.all;
use work.tb_package.all;
use work.dma_package.all;

entity arbiter is
generic (FIRST_CH : integer := 1;
 		 LAST_CH  : integer := 4);
port (		
    -- Clock and reset.
    user_clk		: in std_logic;
    user_reset		: in std_logic;
    
	-- Direct memory access (dma) interface connections to the application logic.
    dma_status		: in DMA_TYPE_4;
	dma_control		: in DMA_TYPE_6;
	ch				: out integer range FIRST_CH to LAST_CH;
    dma_ch_go		: out CH_TYPE_1
	);
end arbiter;   

architecture structural of arbiter is
	signal channel_request	: DMA_TYPE_1; 
	signal channel			: integer range FIRST_CH to LAST_CH;
	signal channel_go		: CH_TYPE_1;
begin                                                          
	-- Every time a request for the bus comes in log the request by channel.
	process (user_reset, dma_control, channel_go)
	begin
		if user_reset = '1' then
			channel_request <= (others => '0');	
		else
			for i in FIRST_CH to LAST_CH loop
				if dma_control(i)(4 downto 0) /= "00000" then
					channel_request(i) <= '1'; 				
				else
					if channel_go(i) = '1' then
						channel_request(i) <= '0'; 					
					end if;
				end if;
			end loop;
		end if;
	end process;

	process (user_clk, user_reset)
	begin
		if user_reset = '1' then
		   	channel_go <= (others => '0');
		   	channel <= FIRST_CH; 
		elsif rising_edge(user_clk) then
			if dma_status(channel) /= "0000" then
				channel_go(channel) <= '0'; 

			elsif channel_request(channel) = '1' then
				channel_go(channel) <= '1';					
				ch <= channel;
				
			elsif channel_go(channel) = '0' then
				if channel = LAST_CH then
					channel <= FIRST_CH;
				else
					channel <= channel + 1;
				end if;
			end if;
		end if;
	end process; 
	     
	dma_ch_go <= channel_go;
end structural;

